package EDU.auburn.VGJ.gui;

import java.util.*;
import java.net.*;
import java.io.*;

public class Modules {

  private int NumberOfDir;
  private int NumberOfMod;
  private Vector DirList;
  private Vector ModList;
  private Hashtable ModHash;
  private Hashtable DirHash;
  private Hashtable shapeHash;
  
  private ModuleDesc curMod;
  private int k;
  private String readString;

  public Modules(String ModulesTxt) {
    DirList = new Vector();
    ModList = new Vector();
    ModHash = new Hashtable();
    DirHash = new Hashtable();

    //System.out.println("URL :"+ModulesTxt);
    
    BufferedReader bufread;
    String sin;
    try{  
      URL url = new URL(ModulesTxt);
      bufread=new BufferedReader(new InputStreamReader(url.openStream()));
      while((sin=bufread.readLine()) != null)
        putInTable(sin);
    }catch (MalformedURLException ev)
      {
        System.out.println("URL error in getting modules.txt: " + ev);
      }
    catch (IOException ev)
      {
        System.out.println("IOException in getting modules.txt:"+ev);
      }

    NumberOfDir = DirList.size();
    NumberOfMod = ModHash.size();
    shapeHash = new Hashtable();
    shapeHash.put("0",new Integer(0));
    shapeHash.put("1",new Integer(2));
    shapeHash.put("2",new Integer(4));
    shapeHash.put("3",new Integer(0));
    shapeHash.put("10",new Integer(5));
    shapeHash.put("11",new Integer(11));
    shapeHash.put("12",new Integer(7));
    shapeHash.put("13",new Integer(8));
    shapeHash.put("20",new Integer(6));
    shapeHash.put("21",new Integer(9));
    shapeHash.put("22",new Integer(10)); 
    shapeHash.put("23",new Integer(0)); 
    shapeHash.put("30",new Integer(0)); 
    shapeHash.put("31",new Integer(0)); 
    shapeHash.put("32",new Integer(0)); 
    shapeHash.put("33",new Integer(3));
  }


public void putInTable(String line)
  {
    //System.out.println("I got line :"+line);
    String module=null,hostPort=null,inPortNum=null,outPortNum=null;
    
    try{
      StringTokenizer st = new StringTokenizer(line," ");
      module             = (String)st.nextToken();
      hostPort           = (String)st.nextToken();
      inPortNum          = (String)st.nextToken();
      outPortNum         = (String)st.nextToken();
    }catch(NoSuchElementException e){
      System.out.println("You have wrong line in modules.txt:"+line);
      return;
    }
    
    
    String className = module;

    int pos = module.lastIndexOf(".");
    String moduleName = module.substring(pos+1);
    
    String subs = module.substring(0,pos);
    pos  = subs.lastIndexOf(".");
    String dirName = subs.substring(pos+1);
    if (dirName.equals("QuantumSimulations"))
      dirName = "QS";

    String server = hostPort;
    
    int In  = Integer.parseInt(inPortNum);
    int Out = Integer.parseInt(outPortNum);
 
    addModule(dirName,moduleName,className,In,Out,server);
    System.err.println(dirName+" "+moduleName+" "+className+" "+In+
    		       " "+Out+" "+server);
    
    
  }
  
    private void addModule(String directory, String name, String Class,
                           int In, int Out, String server) {

     if(DirHash.get(directory) == null) {
          DirList.addElement(directory);
          DirHash.put(directory,"1");
     } 
     
     System.err.println("new ModuleDesc "+name+" "+In+" "+Out);
     ModuleDesc m = new ModuleDesc(name,directory,In,Out,Class);
     m.setServer(server);
     ModList.addElement(directory+"."+name);
     ModHash.put(directory+name,m);
  }


  public String getModuleClass(String ModuleName, String Dir) {
      curMod=(ModuleDesc)ModHash.get(Dir+ModuleName);
      return curMod.getModuleClass();
  }

  public String getModuleServer(String ModuleName, String Dir) {
      curMod=(ModuleDesc)ModHash.get(Dir+ModuleName);
      return curMod.getServer();
  }

  public void setModuleServer(String ModuleName, String Dir, String newServer) {
      curMod=(ModuleDesc)ModHash.get(Dir+ModuleName);
      curMod.setServer(newServer);
    }

  public int getNumberInPorts(String ModuleName, String Dir) {
      curMod=(ModuleDesc)ModHash.get(Dir+ModuleName);
      return curMod.getInPorts();
  }

  public int getNumberOutPorts(String ModuleName, String Dir) {
      curMod=(ModuleDesc)ModHash.get(Dir+ModuleName);
      return curMod.getOutPorts();
  }

  public int getShape(String ModuleName, String Dir) {
      int inport = getNumberInPorts(ModuleName,Dir);
      int outport = getNumberOutPorts(ModuleName,Dir);
      int value = inport*10+outport;
      String pcode = (new Integer(value)).toString(); 
      Integer shape = (Integer)shapeHash.get(pcode);
      int sshape = shape.intValue();
      return sshape;
  }

  public int getNumberOfDir() {
   return NumberOfDir;
  }

  public String getDirName(int i) {
   return (String)DirList.elementAt(i);
  }

  public int getDirSize(String DirName){
   int ds = 0;
   for(k=0;k<NumberOfMod;k++) {
    if(((String)ModList.elementAt(k)).startsWith(DirName)) ds++;
   }
   return ds;
  }

  public Vector getModuleList(String DirName) {
    int idx;
    String ss;
    Vector ml=new Vector();
    for(k=0;k<NumberOfMod;k++) {
      ss = (String)ModList.elementAt(k);
      if(ss.startsWith(DirName)) {
        idx = ss.indexOf(".")+1;
        ml.addElement(ss.substring(idx));
      }
    }
   return ml; 
  }

  public String firstDir(String moduleName) {
    // returns directory of the first occurence of moduleName
   System.err.println("look for "+moduleName);
   int idx;
   String ss;
   String dir=null;
   k=0;
   while (dir==null) {
    ss=(String)ModList.elementAt(k);
    System.err.println("=> "+ss);
       if(ss.endsWith(moduleName)) {
         idx = ss.indexOf(".");
         dir=ss.substring(0,idx);
         System.err.println("found in "+dir); 
       }
    k++;
    if(k > NumberOfMod) break;
   }  
   return dir;
  }


}
